package com.gic.haoban.manage.api.service;

import com.gic.api.base.commons.Page;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.StaffClerkBindLogDetailDTO;
import com.gic.haoban.manage.api.dto.StaffClerkInfoDTO;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;

import java.util.List;
import java.util.Set;

public interface StaffClerkRelationApiService {
    /**
     * 绑定日志
     *
     * @param params
     */
    void bindLogMq(String params);

    List<StaffClerkRelationDTO> listBindCode(String enterpriseId, Set<String> clerkCodeList);

    List<StaffClerkRelationDTO> listBindCodeByStaffId(List<String> enterpriseIdList, String staffId);

    /**
     * 根据staffid 获取列表
     *
     * @param staffId
     * @return
     */
    List<StaffClerkInfoDTO> listBindDetailByStaffId(String staffId);

    /**
     * 绑定
     *
     * @return
     */
    ServiceResponse bindStaffClerk(StaffClerkRelationDTO staffClerkRelationDTO, String optStaffId, int chanelCode);

    StaffClerkRelationDTO getByCodeAndEnterpriseId(String clerkCode, String enterpriseId);


    void delAndInsert(StaffClerkRelationDTO staffClerkRelation, String optStaffId, int chanelCode);

    /**
     * 解绑
     *
     * @param storeId
     * @param clerkCode
     * @return
     */
    String delByStoreIdAndCode(String storeId, String clerkCode);


    StaffClerkRelationDTO getOneByClerkId(String clerkId);

    List<StaffClerkRelationDTO> listByClerkIds(List<String> clerkIds);

    /**
     * 导购列表
     *
     * @param storeId
     * @return
     */
    List<StaffClerkRelationDTO> listByStoreId(String storeId);

    /**
     * 解绑
     *
     * @param staffId 操作人
     * @param clerkId
     * @return
     */
    boolean unbindByStaffAndClerkId(String staffId, String clerkId);

    /**
     * 查询导购信息
     *
     * @param clerkId
     * @return
     */
    StaffClerkRelationDTO getOneByStoreIdAndStaffId(String storeId, String staffId);

    /**
     * 查询导购信息
     *
     * @param clerkId
     * @return
     */
    StaffClerkRelationDTO getByClerkId(String clerkId);

    /**
     * @param enterpriseId
     * @param wxUserId
     * @return
     */
    List<StaffClerkRelationDTO> listByEnterpriseIdAndWxUserId(String enterpriseId, String wxUserId);

    /**
     * 删除关联关系
     *
     * @param clerkId
     */
    void delByClerkId(String clerkId);

    /**
     * 搜索查询列表
     *
     * @param wxEnterpriseId
     * @param search
     * @param enterpriseId
     * @param optType
     * @return
     */
    Page<StaffClerkBindLogDetailDTO> pageStaffClerkBindLog(String wxEnterpriseId, String search, String enterpriseId, int optType, BasePageInfo pageInfo);

    /**
     * 搜索解绑列表
     *
     * @param wxEnterpriseId
     * @param clerkId
     * @param enterpriseId
     * @param pageInfo
     * @return
     */
    Page<StaffClerkBindLogDetailDTO> pageStaffClerkUnBindLog(String wxEnterpriseId, String clerkId, String enterpriseId, int optType, BasePageInfo pageInfo);

    /**
     * 追加到绑定日志
     *
     * @param staffId
     * @param optStaffId
     * @param optType
     * @param channelCode
     * @param relationId
     */
    void pushToBindLog(String staffId, String optStaffId, int optType, int channelCode, String relationId);


    /**
     * 获取config（联系我）
     *
     * @param clerkId
     * @return
     */
    ServiceResponse<String> getClerkMemberChatConfig(String clerkId);

    /**
     * @return 返回值类型: <pre>
     * @author 作者: WQ <pre>
     * @date 时间: 2020年12月24日 上午11:21:38 <pre>
     * @param wxEnterpriseId: <pre>
     * @param pageInfo: <pre>
     * @Description 方法描述:
     */
    Page<StaffClerkRelationDTO> pageByWxEnterpriseId(String wxEnterpriseId, BasePageInfo pageInfo);

    /**
     * 查询状态正常导购
     *
     * @return
     */
    List<StaffClerkRelationDTO> listAll();


    /**
     * 通过员工id列表
     *
     * @param wxEnterpriseId wx企业标识
     * @param staffId        员工id
     * @return {@link List }<{@link StaffClerkRelationDTO }>
     * @author mozhu
     * @date 2022-06-20 22:46:46
     */
    List<StaffClerkRelationDTO> listByStaffId(String wxEnterpriseId,String staffId);
}
