package com.gic.haoban.manage.service.service.out.impl.hm;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.base.api.common.errCode.HaoBanErrCodeCommon;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.hm.HmGroupDTO;
import com.gic.haoban.manage.api.dto.hm.HmGroupQueryDTO;
import com.gic.haoban.manage.api.enums.hm.HmGroupStatus;
import com.gic.haoban.manage.api.service.hm.HmGroupApiService;
import com.gic.haoban.manage.service.dao.mapper.hm.TabHaobanHmGroupSettingMapper;
import com.gic.haoban.manage.service.entity.hm.TabHaobanHmGroupSetting;
import com.gic.haoban.manage.service.pojo.qo.HmGroupInfoQO;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Author MUSI
 * @Date 2022/7/1 4:02 PM
 * @Description
 * @Version
 **/
@Component
public class HmGroupApiServiceImpl implements HmGroupApiService {

    private static final Logger log = LoggerFactory.getLogger(HmGroupApiServiceImpl.class);

    private static final Long DEFAULT_GROUP_ID = NumberUtils.LONG_ZERO;
    private static final String DEFAULT_GROUP_NAME = "默认分组";

    @Autowired
    private TabHaobanHmGroupSettingMapper haobanHmGroupSettingMapper;

    @Override
    public ServiceResponse<Long> saveOrUpdateHmGroupSetting(HmGroupDTO hmGroupDTO) {
        ServiceResponse<Long> serviceResponse = new ServiceResponse();
        if (StringUtils.isBlank(hmGroupDTO.getGroupName())) {
            serviceResponse.setCode(HaoBanErrCodeCommon.ERR_2.getCode());
            return serviceResponse;
        }
        if (StringUtils.equals(DEFAULT_GROUP_NAME, hmGroupDTO.getGroupName())){
            serviceResponse.setCode(HaoBanErrCodeCommon.ERR_5.getCode());
            return serviceResponse;
        }
        HmGroupInfoQO groupInfoQO = new HmGroupInfoQO();
        groupInfoQO.setGroupName(hmGroupDTO.getGroupName());
        groupInfoQO.setStatus(HmGroupStatus.ENABLE.getCode());
        groupInfoQO.setEnterpriseId(hmGroupDTO.getEnterpriseId());
        groupInfoQO.setWxEnterpriseId(groupInfoQO.getWxEnterpriseId());
        List<TabHaobanHmGroupSetting> tabHaobanHmGroupSettings = haobanHmGroupSettingMapper.queryHmGroupSettingList(groupInfoQO);
        if (hmGroupDTO.getGroupId() == null) {
            // insert
            // check groupName unique
            if (CollectionUtils.isNotEmpty(tabHaobanHmGroupSettings)) {
                serviceResponse.setCode(HaoBanErrCodeCommon.ERR_2.getCode());
                return serviceResponse;
            }
            TabHaobanHmGroupSetting tempGroupSetting = new TabHaobanHmGroupSetting();
            tempGroupSetting.setGroupId(UniqueIdUtils.uniqueLong());
            tempGroupSetting.setGroupName(hmGroupDTO.getGroupName());
            tempGroupSetting.setCreatorId(hmGroupDTO.getCreatorId());
            tempGroupSetting.setCreatorName(hmGroupDTO.getCreatorName());
            tempGroupSetting.setCreateTime(new Date());
            tempGroupSetting.setStatus(HmGroupStatus.ENABLE.getCode());
            tempGroupSetting.setEnterpriseId(hmGroupDTO.getEnterpriseId());
            tempGroupSetting.setWxEnterpriseId(hmGroupDTO.getWxEnterpriseId());
            haobanHmGroupSettingMapper.insertSelective(tempGroupSetting);
            serviceResponse.setResult(tempGroupSetting.getGroupId());
        } else {
            // update
            if (CollectionUtils.isNotEmpty(tabHaobanHmGroupSettings)
                    && tabHaobanHmGroupSettings.size() > 1) {
                // 名称重复
                serviceResponse.setCode(HaoBanErrCodeCommon.ERR_5.getCode());
                return serviceResponse;
            }
            if (CollectionUtils.isNotEmpty(tabHaobanHmGroupSettings)
                    && !tabHaobanHmGroupSettings.get(0).getGroupId().equals(hmGroupDTO.getGroupId())) {
                // 名称重复
                serviceResponse.setCode(HaoBanErrCodeCommon.ERR_5.getCode());
                return serviceResponse;
            }
            TabHaobanHmGroupSetting tabHaobanHmGroupSetting = haobanHmGroupSettingMapper.selectByPrimaryKey(hmGroupDTO.getGroupId());
            if (tabHaobanHmGroupSetting == null) {
                serviceResponse.setCode(HaoBanErrCodeCommon.ERR_8.getCode());
                return serviceResponse;
            }
            if (!StringUtils.equals(tabHaobanHmGroupSetting.getEnterpriseId(), hmGroupDTO.getEnterpriseId())) {
                // 无权限
                serviceResponse.setCode(HaoBanErrCodeCommon.ERR_9.getCode());
                return serviceResponse;
            }
            TabHaobanHmGroupSetting tempGroupSetting = new TabHaobanHmGroupSetting();
            tempGroupSetting.setGroupId(tabHaobanHmGroupSetting.getGroupId());
            tempGroupSetting.setGroupName(tabHaobanHmGroupSetting.getGroupName());
            tempGroupSetting.setModifierId(hmGroupDTO.getModifierId());
            tempGroupSetting.setModifierName(hmGroupDTO.getModifierName());
            tempGroupSetting.setUpdateTime(new Date());
            haobanHmGroupSettingMapper.updateByPrimaryKeySelective(tempGroupSetting);
            serviceResponse.setResult(tempGroupSetting.getGroupId());
        }
        // todo 日志
        return serviceResponse;
    }

    @Override
    public ServiceResponse<Page<HmGroupDTO>> queryHmGroupSettingList(HmGroupQueryDTO groupQueryDTO) {

        HmGroupInfoQO groupInfoQo = new HmGroupInfoQO();
        BeanUtils.copyProperties(groupQueryDTO, groupInfoQo);
        groupInfoQo.setStatus(HmGroupStatus.ENABLE.getCode());
        PageHelper.startPage(groupInfoQo.getPageNum(), groupInfoQo.getPageSize());
        com.github.pagehelper.Page<TabHaobanHmGroupSetting> tabHaobanHmGroupSettings =
                (com.github.pagehelper.Page<TabHaobanHmGroupSetting>) haobanHmGroupSettingMapper.queryHmGroupSettingList(groupInfoQo);
        Page<HmGroupDTO> hmGroupInfoPage = PageUtil.changePageHelperToCurrentPage(tabHaobanHmGroupSettings, HmGroupDTO.class);

        // query 活码 默认分组的总数
        List<HmGroupDTO> result = new ArrayList<>();
        result.add(buildDefault(null));
        if (CollectionUtils.isNotEmpty(hmGroupInfoPage.getResult())){
            result.addAll(hmGroupInfoPage.getResult());
        }
        hmGroupInfoPage.setResult(result);
        ServiceResponse<Page<HmGroupDTO>> serviceResponse = new ServiceResponse<>();
        serviceResponse.setResult(hmGroupInfoPage);
        return serviceResponse;
    }

    private HmGroupDTO buildDefault(Integer num){
        HmGroupDTO hmGroupDTO = new HmGroupDTO();
        hmGroupDTO.setGroupId(DEFAULT_GROUP_ID);
        hmGroupDTO.setGroupName(DEFAULT_GROUP_NAME);
        hmGroupDTO.setDefaultFlag(1);
        hmGroupDTO.setReferNum(num);
        return hmGroupDTO;
    }

    @Override
    public ServiceResponse<Boolean> deleteGroupSetting(HmGroupDTO groupDTO) {
        log.info("[deleteGroupSetting] 删除分组, params:{}", JSON.toJSONString(groupDTO));
        ServiceResponse<Boolean> serviceResponse = new ServiceResponse<>();
        serviceResponse.setResult(Boolean.FALSE);
        if (groupDTO.getGroupId() == null) {
            serviceResponse.setCode(HaoBanErrCodeCommon.ERR_2.getCode());
            return serviceResponse;
        }
        if (DEFAULT_GROUP_ID.equals(groupDTO.getGroupId())){
            serviceResponse.setCode(HaoBanErrCodeCommon.ERR_5.getCode());
            return serviceResponse;
        }
        TabHaobanHmGroupSetting tabHaobanHmGroupSetting = haobanHmGroupSettingMapper.selectByPrimaryKey(groupDTO.getGroupId());
        if (tabHaobanHmGroupSetting == null || !HmGroupStatus.ENABLE.getCode().equals(tabHaobanHmGroupSetting.getStatus())) {
            serviceResponse.setCode(HaoBanErrCodeCommon.ERR_5.getCode());
            return serviceResponse;
        }
        // todo query hm 校验分组是否被引用
        TabHaobanHmGroupSetting tempGroupSetting = new TabHaobanHmGroupSetting();
        tempGroupSetting.setGroupId(tabHaobanHmGroupSetting.getGroupId());
        tempGroupSetting.setStatus(HmGroupStatus.DISABLE.getCode());
        tempGroupSetting.setModifierId(groupDTO.getModifierId());
        tempGroupSetting.setModifierName(groupDTO.getModifierName());
        tempGroupSetting.setUpdateTime(new Date());
        serviceResponse.setResult(haobanHmGroupSettingMapper.updateByPrimaryKeySelective(tempGroupSetting) == 1);
        return serviceResponse;
    }

    /**
     * 分组总引用次数
     *
     * @param enterpriseId
     * @param wxEnterpriseId
     * @return
     */
    @Override
    public ServiceResponse<Integer> queryGroupReferCount(String enterpriseId, String wxEnterpriseId) {
        ServiceResponse<Integer> serviceResponse = new ServiceResponse<>();
        serviceResponse.setResult(0);
        return serviceResponse;
    }
}
